<?php defined('SYSPATH') or die('No direct access allowed.');

/**
 * This class extends the core Kohana class by adding some core application
 * specific functions, and configuration.
 *
 * @package    lnApp
 * @subpackage Core
 * @category   Overrides
 * @author     Deon George
 * @copyright  (c) 2010 Deon George
 * @license    http://dev.leenooks.net/license.html
 */
abstract class lnApp_Config extends Kohana {
	/**
	 * Find a list of all database enabled modules
	 *
	 * @uses cache
	 */
	public static function appmodules() {
		$cacheable = TRUE;

		if (array_key_exists('cache',Kohana::modules())) {
			$cache = Cache::instance(static::cachetype());

			if ($cacheable AND $cache->get('modules'))
				return $cache->get('modules');

		} else
			$cache = '';

		$modules = array();
		$module_table = 'module';

		if (class_exists('Model_'.ucfirst($module_table))) {
			$mo = ORM::factory($module_table)->where('status','=',1)->find_all()->as_array();

			foreach ($mo as $o)
				$modules[$o->name] = MODPATH.$o->name;
		}

		if ($cache)
			$cache->set('modules',$modules);

		return $modules;
	}

	/**
	 * Return our site name
	 */
	public static function site() {
		if (! empty($_SERVER['SERVER_NAME']))
			return $_SERVER['SERVER_NAME'];

		if (! $site = CLI::options('site'))
			throw new Kohana_Exception(_('Cant figure out the site, use --site= for CLI'));

		return $site['site'];
	}

	/**
	 * Work out our site ID for multiehosting
	 * @todo Change this to query the DB for site number.
	 */
	public static function siteid() {
		$sites = Kohana::config('config.site');

		// If we havent been configured for sites
		if (is_null($sites) OR ! is_array($sites) OR ! isset($sites[static::site()]))
			return 0;
		else
			return $sites[static::site()];
	}

	/**
	 * Work out our site mode (dev,test,prod)
	 * @todo Change this to query the DB for mode.
	 */
	public static function sitemode() {
		$sites = Kohana::config('config.site_mode');

		// If we havent been configured for sites
		if (is_null($sites) OR ! is_array($sites) OR ! isset($sites[static::site()]))
			return Kohana::PRODUCTION;
		else
			return $sites[static::site()];
	}

	public static function sitename() {
		return Kohana::config('config.site_name');
	}

	public static function logo() {
		$mediapath = Route::get('default/media');
		$logo = $mediapath->uri(array('file'=>'img/logo-small.png'),array('alt'=>static::sitename()));

		return HTML::image($logo,array('class'=>'headlogo','alt'=>_('Logo')));
	}

	/**
	 * Return our caching mechanism
	 */
	public static function cachetype() {
		return is_null(Kohana::config('config.cache_type')) ? 'file' : Kohana::config('config.cache_type');
	}

	/**
	 * Show a date using a site configured format
	 */
	public static function date($date) {
		return date(Kohana::config('config.date_format'),$date);
	}

	/**
	 * See if our emails for the template should be sent to configured admin(s)
	 *
	 * @param string template - Template to test for
	 * @return mixed|array - Email to send test emails to
	 */
	public static function testmail($template) {
		$config = Kohana::config('config.email_admin_only');

		if (is_null($config) OR ! is_array($config) OR empty($config[$template]))
			return FALSE;
		else
			return $config[$template];
	}
}
?>
